import glob
import os
import sys
from typing import Any, List, Optional

import MeCab
from gensim.models import Word2Vec  # type: ignore


class AozoraVecMaker():
    __pwd__ = os.getcwd()
    STOPWORDS = ['これ', 'それ', 'あれ', 'この', 'その',
                 'あの', 'ここ', 'そこ', 'あそこ', 'こちら',
                 'どこ', 'だれ', 'なに', 'なん', '何',
                 '私', '貴方', '貴方方', '我々', '私達',
                 'あの人', 'あのかた', '彼女', '彼', 'です',
                 'あります', 'おります', 'います', 'は', 'が',
                 'の', 'に', 'を', 'で', 'え',
                 'から', 'まで', 'より', 'も', 'どの',
                 'と', 'し', 'それで', 'しかし', '',
                 *[chr(i) for i in range(12354, 12436)]]

    def __init__(self, novel_dir: str = 'novel') -> None:
        self.novel_dir = os.path.join(self.__pwd__, novel_dir)
        if not os.path.exists(self.novel_dir):
            raise FileExistsError(self.novel_dir)
        sys.stdout = sys.stderr = open(os.devnull, 'w')
        self.__tagger = self.__make_tagger()
        sys.stdout, sys.stderr = sys.__stdout__, sys.__stderr__

    def make_model(self, save_modelname: str = 'aozora_model',
                   epochs: int = 5, vector_size: int = 100,
                   min_count: int = 5, window: int = 5, workers: int = 3,
                   binary: bool = False, both: bool = False) -> None:
        def t(line: str) -> List[Any]:
            return self.tokenizer(line, part_use=None)

        all_novel_lines = []
        ps = glob.glob(os.path.join(self.novel_dir, '*', '*'))
        ps_len = len(ps)
        for idx, p in enumerate(ps):
            print('{}/{}'.format(idx+1, ps_len), end='\r', file=sys.stderr)
            f = open(p, 'r')
            novel_content = f.read()
            novel_text_lines = novel_content.split('\n')

            tokenized_novel_text_lines = [t(line)
                                          for line in novel_text_lines]
            all_novel_lines.extend(tokenized_novel_text_lines)

        model = Word2Vec(all_novel_lines, epochs=epochs,
                         vector_size=vector_size,
                         min_count=min_count, window=window, workers=workers)
        self.__save_model(model, save_modelname, binary, both)

    def __save_model(self, model: Word2Vec, save_modelname: str,
                     binary: bool, both: bool) -> None:
        model_name = os.path.join(self.__pwd__, save_modelname + '.model')
        kv_name = os.path.join(self.__pwd__, save_modelname + '.kv')

        if both:
            model.save_word2vec_format(model_name + '.bin', binary=True)
            model.save_word2vec_format(model_name)
            model.wv.save_word2vec_format(kv_name + '.bin', binary=True)
            model.wv.save_word2vec_format(kv_name)
            return None

        if binary:
            model.save_word2vec_format(model_name + '.bin', binary=True)
        else:
            model.save_word2vec_format(model_name)

        if binary:
            model.wv.save_word2vec_format(kv_name + '.bin', binary=True)
        else:
            model.wv.save_word2vec_format(kv_name)

    def tokenizer(self, words: str,
                  part_use: Optional[List[str]] = ['名詞', '形容詞']) -> List[Any]:
        self.__tagger.parse('')
        nodes: MeCab.Tagger = self.__tagger.parseToNode(words)
        tokenized = []
        while nodes:
            word = nodes.surface
            pos = nodes.feature.split(',')
            part = pos[0]
            if len(part) >= 7:
                word = pos[6]

            if self.__is_word(word, part, part_use):
                tokenized.append(word)

            nodes = nodes.next

        return tokenized

    @classmethod
    def __is_word(cls, word: str, part: str,
                  part_use: Optional[List[str]]) -> bool:
        is_stop = word not in cls.STOPWORDS
        is_part = part is None or part_use is None
        return is_stop and (is_part or (type(part_use) is list and
                                        part in part_use and
                                        not word.isalnum()))

    @staticmethod
    def __make_tagger() -> MeCab.Tagger:
        try:
            tagger = MeCab.Tagger('')
        except RuntimeError:
            tagger = MeCab.Tagger('-r/etc/mecabrc')

        return tagger


if __name__ == '__main__':
    AozoraVecMaker().make_model()



import os
import subprocess
import sys
from shutil import which
from typing import Optional


class GitIsNotInstalled(Exception):
    pass


class AozoraCloner():
    url = 'https://github.com/aozorabunko/aozorabunko'

    def __init__(self) -> None:
        pass

    @classmethod
    def get_url(cls) -> str:
        return cls.url

    @classmethod
    def set_url(cls, url: str) -> None:
        cls.url = url

    @classmethod
    def clone(cls) -> Optional[int]:
        if not which('git'):
            raise GitIsNotInstalled
        else:
            p = subprocess.Popen('git clone --depth 1 ' + cls.url, shell=True)
            p.wait()
            return p.poll()


if __name__ == '__main__':
    REPO = os.path.join(os.getcwd(), 'aozorabunko')

    if os.path.isdir(REPO) and os.path.isdir(os.path.join(REPO, '.git')):
        print('aozorabunko already exists!', file=sys.stderr)
        exit(0)
    else:
        AozoraCloner().clone()



from setuptools import find_packages, setup  # type: ignore

from aovec import __version__

setup(
    name='aovec',
    version=__version__,
    description='Make Word2Vec from aozorabunko/aozorabunko',
    description_content_type='',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/eggplants/aovec',
    author='eggplants',
    packages=find_packages(),
    python_requires='>=3.5',
    include_package_data=True,
    license='MIT',
    install_requires=open('requirements.txt').read().rstrip().split('\n'),
    entry_points={
        'console_scripts': [
            'aovec=aovec.main:main'
        ]
    }
)



import argparse
from argparse import Namespace as NS
from typing import List, Optional

from aovec import __version__

from .AozoraCloner import AozoraCloner
from .AozoraParser import AozoraParser
from .AozoraVecMaker import AozoraVecMaker


def command_clone(args: NS) -> None:
    AozoraCloner().clone()


def command_parse(args: NS) -> None:
    AozoraParser(args.savedir).parse()


def command_mkvec(args: NS) -> None:
    AozoraVecMaker(args.parsedir).make_model(
        save_modelname=args.model, epochs=args.epochs,
        vector_size=args.vector_size, min_count=args.min_count,
        window=args.window, workers=args.workers,
        binary=args.binary, both=args.both)


def check_positive(v: str) -> int:
    if int(v) <= 0:
        raise argparse.ArgumentTypeError(
            "%s is an invalid natural number" % int(v))
    return int(v)


def make_argparser() -> argparse.ArgumentParser:
    """Parse arguments."""

    parser = argparse.ArgumentParser(
        prog='aovec',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        description='Make Word2Vec from aozorabunko/aozorabunko')

    parser.add_argument('-V', '--version', action='version',
                        version='%(prog)s {}'.format(__version__))

    subparsers = parser.add_subparsers()

    parser_clone = subparsers.add_parser(
        'clone', aliases=['c'], help='clone aozorabunko/aozorabunko (>20GB)',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,)
    parser_clone.set_defaults(handler=command_clone)

    parser_parse = subparsers.add_parser(
        'parse', aliases=['p'], help='parse html files and write to results',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,)
    parser_parse.add_argument(
        '-d', '--savedir', default='novels',
        metavar='DIR', help='directory name of saving results')
    parser_parse.set_defaults(handler=command_parse)

    parser_mkvec = subparsers.add_parser(
        'mkvec', aliases=['m'], help='make word2vec and write to *.model',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,)
    parser_mkvec.add_argument(
        '-d', '--parsedir', default='novels',
        metavar='DIR', help='directory name of saved parsing results')
    parser_mkvec.add_argument(
        '-o', '--model', default='aozora_model',
        metavar='NAME', help='name of word2vec model')
    parser_mkvec.add_argument(
        '-e', '--epochs', default=5, type=check_positive,
        metavar='INT', help='number of word2vec epochs')
    parser_mkvec.add_argument(
        '-v', '--vector_size', default=1000, type=check_positive,
        metavar='INT', help='dimensionality of the word vectors')
    parser_mkvec.add_argument(
        '-m', '--min_count', default=5, type=check_positive,
        metavar='INT', help='ignore words total frequency lower than this')
    parser_mkvec.add_argument(
        '-w', '--window', default=5, type=check_positive,
        metavar='INT', help='window size of words before and for learning')
    parser_mkvec.add_argument(
        '-p', '--workers', default=3, type=check_positive,
        metavar='INT', help='worker threads')
    parser_mkvec.add_argument(
        '-b', '--binary', action='store_true',
        help='save model files as one binary')
    parser_mkvec.add_argument(
        '--both', action='store_true',
        help='save model files as both row data and binary')

    parser_mkvec.set_defaults(handler=command_mkvec)

    return parser


def parse(test: Optional[List[str]] = None) -> None:
    parser = make_argparser()
    args = parser.parse_args(test)

    if hasattr(args, 'handler'):
        args.handler(args)
    else:
        parser.print_help()


def main() -> None:
    parse()


if __name__ == '__main__':
    main()



import codecs
import glob
import os
import sys
from typing import List, Optional, Tuple

from bs4 import BeautifulSoup as bs


class AozoraParser():
    __pwd__ = os.getcwd()
    AOZORA_BASE = os.path.join(__pwd__, 'aozorabunko')
    AOZORA_CARDS = os.path.join(AOZORA_BASE, 'cards')
    AOZORA_FILES = os.path.join(AOZORA_CARDS, '{}', 'files')

    def __init__(self, savedirname: str = 'novels') -> None:
        self.SAVEDIR = os.path.join(self.__pwd__, savedirname)
        os.makedirs(self.SAVEDIR, exist_ok=True)
        self.__check_aozora_dir()

    @classmethod
    def __check_aozora_dir(cls) -> None:
        if not cls.check_exist(cls.AOZORA_CARDS):
            raise FileNotFoundError(cls.AOZORA_CARDS)

    def parse(self) -> None:
        cards = [self.get_novels(c) for c in self.get_cards()]
        c_len = len(sum(cards, []))
        idx = 0
        for novels in cards:
            for novel in novels:
                res = self.__parse_novel(novel)
                # print(res[0:2] if res is not None else '')
                idx += 1
                print('{}/{}'.format(idx, c_len), end='\r', file=sys.stderr)
                if res:
                    sd = os.path.join(self.SAVEDIR, res[0])
                    sf = os.path.join(sd, res[1])
                    os.makedirs(sd, exist_ok=True)
                    self.__save_text(sf, res[2])

    @classmethod
    def __parse_novel(cls, novel: str) -> Optional[Tuple[str, str, str]]:
        # print(novel)
        f = codecs.open(novel, 'r', 'shiftjis', 'ignore')
        source = bs(f.read(), 'html.parser')
        cls.__decompose(source)

        novel_title = source.find('h1', class_='title')
        novel_author = source.find('h2', class_='author')
        novel_content = source.find('div', class_='main_text')
        f1 = novel_author is not None
        if f1 and novel_author.string is not None:
            return (cls.__shrink(novel_author.string),
                    novel_title.string,
                    novel_content.text)
        else:
            return None

    @staticmethod
    def __shrink(t: str, length: int = 100) -> str:
        t = t.replace('　', '_')
        if len(t) > 100:
            return t[0:100] + '…'
        else:
            return t

    @staticmethod
    def __save_text(path: str, text: str) -> None:
        print(text, file=open(path, 'w'))

    @staticmethod
    def __decompose(bs_: bs) -> None:
        try:
            for rt in bs_('rt'):
                rt.decompose()
            for rp in bs_('rp'):
                rp.decompose()
        except AttributeError as err:
            print(err)

    @staticmethod
    def check_exist(path: str) -> bool:
        return os.path.exists(path)

    @classmethod
    def get_cards(cls) -> List[str]:
        return [c for c in os.listdir(cls.AOZORA_CARDS)
                if c.isdecimal()]

    @classmethod
    def get_novels(cls, card: str) -> List[str]:
        f = cls.AOZORA_FILES.format(card)
        return glob.glob(os.path.join(f, '*.html'))


if __name__ == '__main__':
    AozoraParser().parse()
